"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadNpm = exports.installNpm = exports.getCoreBaseDir = void 0;
const path_1 = require("path");
const fs_1 = require("fs");
const child_process_1 = require("child_process");
const assert = require("assert");
const getCoreBaseDir = () => {
    return child_process_1.execSync('npm root').toString().replace(/\n$/, '');
};
exports.getCoreBaseDir = getCoreBaseDir;
async function getNpmPath(scope, npmName, npmRegistry) {
    const currentNodeModules = exports.getCoreBaseDir();
    const localNpmPath = path_1.join(currentNodeModules, npmName);
    if (fs_1.existsSync(localNpmPath)) {
        return localNpmPath;
    }
    let baseDir = path_1.join(currentNodeModules, '../');
    if (!fs_1.existsSync(baseDir)) {
        baseDir = process.cwd();
    }
    const pkgJson = path_1.join(baseDir, 'package.json');
    if (!fs_1.existsSync(pkgJson)) {
        fs_1.writeFileSync(pkgJson, '{}');
    }
    scope.coreInstance.cli.log(`Installing ${npmName}`);
    await installNpm({
        baseDir,
        register: npmRegistry,
        npmName,
        mode: 'production --no-save',
    });
    return path_1.join(baseDir, `node_modules/${npmName}`);
}
async function installNpm(options) {
    const { baseDir, register = 'npm', npmName, mode, slience, registerPath, } = options;
    const cmd = `${register} i ${npmName}${mode ? ` --${mode}` : ' --no-save'}${registerPath ? ` --registry=${registerPath}` : ''}`;
    return new Promise((resolved, rejected) => {
        const execProcess = child_process_1.exec(cmd, {
            cwd: baseDir,
        }, (err, result) => {
            if (err) {
                return rejected(err);
            }
            resolved(result.replace(/\n$/, '').replace(/^\s*|\s*$/, ''));
        });
        execProcess.stdout.on('data', data => {
            if (!slience) {
                console.log(data);
            }
        });
    });
}
exports.installNpm = installNpm;
async function loadNpm(scope, npmName, npmRegistry) {
    try {
        const npmPath = await getNpmPath(scope, npmName, npmRegistry);
        assert(npmPath, 'empty npm path');
        const plugin = require(npmPath);
        scope.addPlugin(plugin);
    }
    catch (e) {
        // scope.error('npmPlugin', { npmName, err: e });
    }
}
exports.loadNpm = loadNpm;
//# sourceMappingURL=npm.js.map